---@type Mq
mq = require('mq')

-- Do you hate levitation?
-- Do you find yourself miles above the zone? 
-- Are you so far up that you can't see any terrain?
-- Are you actually trying to get into or under something?
-- Are you bobbing around like a turd that won't flush?
-- 
-- Then please allow me to introduce iHateLevitate, the solution to keep your feet firmly on the ground.
-- 
-- Notes:
-- If you use option 1 (Stop Floating Now), 
-- then it will seek and destroy any and all "Beneficial" buffs with the dreaded attribute 57 (levitate).
-- It does not care if that is a once in lifetime buff that also gives a million hp, 10x spell damage, and 10x XP/AA.

local args = {...}


if #args == 0 then
    print('iHateLevitate Usage:')
    print('  Stop Floating Now:')
    print('    /lua run iHateLevitate 1')
    print('  Block Common Levitation Spells:')
    print('    /lua run iHateLevitate 2')
    print('  Unblock Levitation Spells:')
    print('    /lua run iHateLevitate 0')
    print('--- FOR PETS ---')
    print('  Block Common Levitation Spells for PETS:')
    print('    /lua run iHateLevitate 3')
    print('  Unblock Levitation Spells for PETS:')
    print('    /lua run iHateLevitate 4')
    return
end


-- Way more spells than can possibly fit in your Blocked Buff list
-- Used to remove levitation spells from the blocked list
-- Probably Incomplete
local levitate_spells = {
    '261', -- Levitate
    '457', -- Dead Man Floating
    '825', -- Whirl Bolt
    '970', -- Levitation
    '1391', -- Dead Men Floating
    '1742', -- Bobbing Corpse
    '2517', -- Spirit of Eagle
    '2566', -- Trickster's Augmentation
    '2894', -- Levitation
    '3185', -- Flight of Eagles
    '3581', -- O'Keil's Levity
    '3974', -- Levity
    '4241', -- Weightless Mist
    '6578', -- Mass Illusion Imp
    '6580', -- Mass Illusion Air Elemental
    '7530', -- Flight
    '7544', -- Levitation
    '8776', -- Vortex Eddy
    '10552', -- Funeral Procession
    '13123', -- Lucky Man Floating
    '13994', -- Group Perfected Levitation
    '21059', -- Bottle of Buoyancy
    '21748', -- Bobbing Corpse I
    '21763', -- Dead Man Floating
    '26828', -- Levitation
    '27722', -- Illusion Aviak Rook
    '27737', -- Illusion Eagle Aviak
    '37894', -- Gravity is for Suckers
    '38686', -- Nightmare
    '39014', -- Fizzy Lifting Drink
    '50082', -- Spirit of Falcons
    '50131', -- Flight of Falcons
    '50655', -- Familiar Crimson Spiroc
    '27746', -- Illusion Vitrik
    '38094', -- Noteworthy Disguise Blue Drake II
    '48823', -- Shauri's Levitation I
    '718', -- Agilmente's Aria of Eagles
    '779', -- Bye-Bye
    '939', -- Eye Levitate
    '1210', -- Bye-Bye
    '1328', -- Agilmente's Aria of Eagles
    '12440', -- Knowledge of Flight
    '21638', -- Nimbus of the Cloudwalker
    '30962', -- Resplendence
    '32789', -- Weightless Aura
    '32870', -- Eye Levitate
    '33100', -- Silent Steps
    '39257', -- Otherworldly Protection
    '39852', -- Guide Levitation
    '41010', -- Suspended Sentence
    '46271', -- Eye Levitate
    '1330', -- Selo's Song of Travel
    '1750', -- Selo's Song of Travel
    '2007', -- Selo's Song of Travel
    '10410', -- Selo's Sonorous Speedsong
    '13125', -- Happy Feet
    '50191', -- Selo's Accelerating Canto
    '50638', -- Familiar Bixie
    '12331', -- Burn of the Flame Telmira
    '12405', -- Kedge Peace
    '33578', -- Froglok Phantom Power
    '33777', -- Fairy's Spell
    '33890', -- Bixie Defense
    '37789', -- Downpour of the Flood Telmira
    '40624', -- Illusion Benefit Levitation
    '50624', -- Familiar Steamwork Flier
}

local common_levitate_spells = {
    '261', -- Levitate
    '457', -- Dead Man Floating
    '970', -- Levitation
    '1391', -- Dead Men Floating
    '2517', -- Spirit of Eagle
    '2894', -- Levitation
    '3185', -- Flight of Eagles
    '3581', -- O'Keil's Levity
    '3974', -- Levity
    '7530', -- Flight
    '13994', -- Group Perfected Levitation
    '50082', -- Spirit of Falcons
    '50131', -- Flight of Falcons
    '1330', -- Selo's Song of Travel
    '10410', -- Selo's Sonorous Speedsong
    '50191', -- Selo's Accelerating Canto
    '48823', -- Shauri's Levitation I
}

function BlockSpells(spells, action, target)
    for _, value in pairs(spells) do
        -- print(value)
        mq.cmd("/blockspell " .. action .. " " .. target .. " " .. value)
        mq.delay(50)
    end
end

local levitate_attrib = 57

function WhyAmIFloating()
    -- figure out why am i floating and stop it
    if not mq.TLO.Me.BuffsPopulated() then
		mq.TLO.Me.DoTarget()
		mq.delay(500)
	end
    local is_levitate = false
    for i = 1,42 do
        if mq.TLO.Me.Buff(i)() ~= nil then
            local buff = mq.TLO.Me.Buff(i)
            local spell = buff.Spell
            if spell.Beneficial() then
                is_levitate = false
                -- print(i .. ' ' .. spell() .. ' ' .. spell.NumEffects())
                for j = 1,buff.NumEffects() do
                    if buff.Attrib(j)() == levitate_attrib then
                        is_levitate = true
                        break
                    end
                end
                if is_levitate then
                    print('iHateLevitate: blocking ' .. spell.ID() .. ' ' .. spell())
                    buff.Remove()
                    mq.cmd("/blockspell add me " .. spell.ID())
                    mq.delay(50)
                end
            end
        end
    end
end

if args[1] == '0' then
    print('iHateLevitate Unblocking Levitate Spells')
    BlockSpells(levitate_spells, 'remove', 'me')
elseif args[1] == '1' then
    WhyAmIFloating()
elseif args[1] == '2' then
    print('iHateLevitate Blocking Common Levitate Spells')
    BlockSpells(common_levitate_spells, 'add', 'me')
elseif args[1] == '3' then
    print('iHateLevitate Blocking Common Levitate Spells for PETS')
    BlockSpells(common_levitate_spells, 'add', 'pet')
elseif args[1] == '4' then
    print('iHateLevitate Unblocking Levitate Spells for PETS')
    BlockSpells(levitate_spells, 'remove', 'pet')
end
